import { ne, relations } from "drizzle-orm";
import {
	check,
	integer,
	numeric,
	pgTable,
	text,
	timestamp,
	varchar,
} from "drizzle-orm/pg-core";

export const users = pgTable("users", {
	id: integer().primaryKey().generatedAlwaysAsIdentity(),
	username: varchar({ length: 128 }).unique().notNull(),
	passwordHash: text(),
	displayName: text(),
});

export const userRelations = relations(users, ({ many }) => ({
	accounts: many(accounts),
}));

export const sessions = pgTable("sessions", {
	id: text().primaryKey(),
	userId: integer()
		.notNull()
		.references(() => users.id),
	expiresAt: timestamp({ withTimezone: true }).notNull(),
});

export const sessionRelations = relations(sessions, ({ one }) => ({
	user: one(users, {
		fields: [sessions.userId],
		references: [users.id],
	}),
}));

export const accounts = pgTable("accounts", {
	id: integer().primaryKey().generatedAlwaysAsIdentity(),
	ownerId: integer()
		.notNull()
		.references(() => users.id),
	name: text().notNull(),
	currency: text().notNull(),
	startingDate: timestamp({ withTimezone: true }).notNull(),
	startingBalance: numeric().notNull().default("0"),
});

export const accountRelations = relations(accounts, ({ one, many }) => ({
	owner: one(users, {
		fields: [accounts.ownerId],
		references: [users.id],
	}),
	credits: many(receipts, { relationName: "account_from" }),
	debits: many(receipts, { relationName: "account_to" }),
}));

export const receipts = pgTable(
	"receipts",
	{
		id: integer().primaryKey().generatedAlwaysAsIdentity(),
		date: timestamp({ withTimezone: true }).notNull(),

		accountFromId: integer()
			.notNull()
			.references(() => accounts.id),

		accountToId: integer().references(() => accounts.id),
	},
	(table) => [
		check("no_account_loops", ne(table.accountFromId, table.accountToId)),
	],
);

export const receiptRelations = relations(receipts, ({ one, many }) => ({
	accountFrom: one(accounts, {
		fields: [receipts.accountFromId],
		references: [accounts.id],
		relationName: "account_from",
	}),
	accountTo: one(accounts, {
		fields: [receipts.accountToId],
		references: [accounts.id],
		relationName: "account_to",
	}),
	receipts: many(receipts),
}));

export const receiptItems = pgTable("receipt_items", {
	id: integer().primaryKey().generatedAlwaysAsIdentity(),
	receiptId: integer()
		.notNull()
		.references(() => receipts.id),
	amount: numeric().notNull().default("0"),
	category: text().notNull(),
});

export const receiptItemRelations = relations(receiptItems, ({ one }) => ({
	receipt: one(receipts, {
		fields: [receiptItems.receiptId],
		references: [receipts.id],
	}),
}));
